/* DSP LABORATORY 
 * University of Technology Graz, 2018
 * Josef Kulmer
 *
 * Obtained from 
 * V1.0  Oct.02, 2005,  C.Feldbauer <feldbauer@tugraz.at>
 * 
 * class Quam contains random symbol generation and its encoding using QAM
 */

#ifndef QUAM_H
#define QUAM_H

#include <math.h>
#include <vector>
#include <stdlib.h>

// carrier frequency in Hz
#define CARRIERFREQ 8000
#define MODULATE 0
#define SINGLECHANNEL 0

// if INC_TRANS_CHANNEL==0 then communication channel is not simulated
// if INC_TRANS_CHANNEL==1 a simple constant gain is simulated
// if INC_TRANS_CHANNEL==2 a time-varying constant gain is simulated
#define INC_TRANS_CHANNEL 0

// number of samples per symbol 
#define  N 16 // defines the symbol rate

// modulation scheme used 4-QAM, 16-QAM, 64-QAM, 256-QAM, or 1024-QAM   
#define  QAM 4  

// Number of bits to generate
// this of course depends on the modulation scheme used
#if QAM==4
	#define NBits 2
	#define POW  1
#endif	
#if QAM==16
	#define NBits 4
	#define POW  2
#endif	
#if QAM==64
	#define NBits 6
	#define POW  3
#endif
#if QAM==256
	#define NBits 8
	#define POW  4
#endif
#if QAM==1024
	#define NBits 10
	#define POW  5
#endif
#define RCosFilt 33

class Quam{
private:
    int sync; // amplitude of the sync signal 
    float A; // Symbol amplitude
    int counter;// counter of output samples (samples per symbol)
    int n;// time index for the carrier generation
    int CHn;// time index for the time varying channel
    float pi;
    float  z_I[RCosFilt], z_Q[RCosFilt];// delay line for the FIR filters  
    int curr_I, curr_Q;  // pointers to the head of the FIR delay lines 
    int b[NBits];// random data bits (in words) before mapping
    float I, Q;// inphase and quadrature 
    float OutI, OutQ; 
    float SinCarrier, CosCarrier;// carriers
    // filter designed in MATLAB with rcosfir() function
    // Design parameters: Roll-off=0.5, N_T=1, RATE=16 
    float G[RCosFilt] = {0,
        0.0536,
        0.1159,
        0.1859,
        0.2625,
        0.3441,
        0.4290,
        0.5151,
        0.6002,
        0.6822,
        0.7587,
        0.8278,
        0.8872,
        0.9354,
        0.9709,
        0.9927,
        1.0000,
        0.9927,
        0.9709,
        0.9354,
        0.8872,
        0.8278,
        0.7587,
        0.6822,
        0.6002,
        0.5151,
        0.4290,
        0.3441,
        0.2625,
        0.1859,
        0.1159,
        0.0536,
        0};
    // convolution of symbols with baseband signal
    float fir(int *, float , int , float [], float []);
    // put symbols to I and Q
    void QAMmap();
    // simulate time varying channel
    void channel();
    // generate random symbols
    void QAMmod();
    // add carrier signal to I and Q
    void carrier();
public:
    Quam();
    ~Quam();
    // perform QAM for a single sample
    void qam();
    // return outcome of QAM I
    float processing_l();
    // return outcome of QAM I
    float processing_r();
};

#endif /* QUAM_H */

